#ifndef HDC1080_H
#define HDC1080_H

#include <stdint.h>
#include <stddef.h>

#define HDC1080_I2C_7BIT_ADDRESS	0x40
#define HDC1080_I2C_READ_ADDRESS	((HDC1080_I2C_7BIT_ADDRESS << 1) | 0x01)
#define HDC1080_I2C_WRITE_ADDRESS	((HDC1080_I2C_7BIT_ADDRESS << 1) | 0x00)

#define HDC1080_REG_TEMP			0x00
#define HDC1080_REG_HUMI			0x01
#define HDC1080_REG_CONFIG			0x02
#define HDC1080_REG_ID1				0xFB
#define HDC1080_REG_ID2				0xFC
#define HDC1080_REG_ID3				0xFD
#define HDC1080_REG_IDMFG			0xFE
#define HDC1080_REG_IDPROD			0xFF

#define HDC1080_EXPAND_MACRO(content)		content
#define HDC1080_FIELD(NAME)				NAME##_MASK, NAME##_OFFSET
#define HDC1080_GET_FIELD_IMPL(FIELD_MASK,FIELD_OFFSET,value)	(((value) >> FIELD_OFFSET) & FIELD_MASK)
#define HDC1080_SET_FIELD_IMPL(FIELD_MASK,FIELD_OFFSET,value)	(((value) & FIELD_MASK) << FIELD_OFFSET)
#define HDC1080_GET_FIELD(...)				HDC1080_EXPAND_MACRO(HDC1080_GET_FIELD_IMPL(__VA_ARGS__))
#define HDC1080_SET_FIELD(...)				HDC1080_EXPAND_MACRO(HDC1080_SET_FIELD_IMPL(__VA_ARGS__))

/* CONFIG Fields */
#define HDC1080_CONFIG_HRES_MASK	0x3
#define HDC1080_CONFIG_HRES_OFFSET	8
#define HDC1080_CONFIG_HRES_FIELD	HDC1080_FIELD(HDC1080_CONFIG_HRES)

#define HDC1080_CONFIG_TRES_MASK	0x1
#define HDC1080_CONFIG_TRES_OFFSET	10
#define HDC1080_CONFIG_TRES_FIELD	HDC1080_FIELD(HDC1080_CONFIG_TRES)

#define HDC1080_CONFIG_BTST_MASK	0x1
#define HDC1080_CONFIG_BTST_OFFSET	11
#define HDC1080_CONFIG_BTST_FIELD	HDC1080_FIELD(HDC1080_CONFIG_BTST)

#define HDC1080_CONFIG_MODE_MASK	0x1
#define HDC1080_CONFIG_MODE_OFFSET	12
#define HDC1080_CONFIG_MODE_FIELD	HDC1080_FIELD(HDC1080_CONFIG_MODE)

#define HDC1080_CONFIG_HEAT_MASK	0x1
#define HDC1080_CONFIG_HEAT_OFFSET	13
#define HDC1080_CONFIG_HEAT_FIELD	HDC1080_FIELD(HDC1080_CONFIG_HEAT)

#define HDC1080_CONFIG_RST_MASK		0x1
#define HDC1080_CONFIG_RST_OFFSET	15
#define HDC1080_CONFIG_RST_FIELD	HDC1080_FIELD(HDC1080_CONFIG_RST)

#define HDC1080_TEMP_11BIT_TIME_US	3650
#define HDC1080_TEMP_14BIT_TIME_US	6350
#define HDC1080_HUMI_8BIT_TIME_US	2500
#define HDC1080_HUMI_11BIT_TIME_US	3850
#define HDC1080_HUMI_14BIT_TIME_US	6500

typedef enum {
	HDC1080_Status_Ok = 0,
	HDC1080_Status_I2CNack = -10,
	HDC1080_Status_I2CError = -11,
	HDC1080_Status_I2CTimeout = -12,
	HDC1080_Status_I2CArbitrationLost = -14,
	HDC1080_Status_NotImplemented = -20,
	HDC1080_Status_BadArg = -21,
	HDC1080_Status_InvalidOperation = -22,
	HDC1080_Status_NotSupported = -24,
} HDC1080_Status;

typedef enum {
	HDC1080_Mode_MeasureTemperatureOrHumidity = 0,
	HDC1080_Mode_MeasureTemperatureThenHumidity = 1,
} HDC1080_Mode;

typedef enum {
	HDC1080_TemperatureResolution_14bit = 0,
	HDC1080_TemperatureResolution_11bit = 1,
} HDC1080_TemperatureResolution;

typedef enum {
	HDC1080_HumidityResolution_14bit = 0,
	HDC1080_HumidityResolution_11bit = 1,
	HDC1080_HumidityResolution_8bit = 2,
} HDC1080_HumidityResolution;

typedef enum {
	HDC1080_BatteryStatus_Ok = 0,
	HDC1080_BatteryStatus_Low = 1,
} HDC1080_BatteryStatus;

typedef struct {
	int enableHeater;
	HDC1080_Mode mode;
	HDC1080_TemperatureResolution temperatureResolution;
	HDC1080_HumidityResolution humidityResolution;
} HDC1080_Configuration;

HDC1080_Status HDC1080_Init();
HDC1080_Status HDC1080_Deinit();

HDC1080_Status HDC1080_GetDefaultConfiguration(HDC1080_Configuration* config);
HDC1080_Status HDC1080_GetConfiguration(HDC1080_Configuration* config);
HDC1080_Status HDC1080_SetConfiguration(HDC1080_Configuration* config);

HDC1080_Status HDC1080_TriggerTemperatureAndHumidityConversion();
HDC1080_Status HDC1080_TriggerTemperatureConversion();
HDC1080_Status HDC1080_TriggerHumidityConversion();

HDC1080_Status HDC1080_GetRawTemperature(uint16_t* temperature);
HDC1080_Status HDC1080_GetRawHumidity(uint16_t* humidity);
HDC1080_Status HDC1080_GetRawTemperatureAndHumidity(uint16_t* temperature, uint16_t* humidity);

HDC1080_Status HDC1080_GetTemperature(float* temperature);
HDC1080_Status HDC1080_GetHumidity(float* humidity);
HDC1080_Status HDC1080_GetTemperatureAndHumidity(float* temperature, float* humidity);

HDC1080_Status HDC1080_EnableHeater();
HDC1080_Status HDC1080_DisableHeater();

HDC1080_Status HDC1080_GetBatteryStatus(HDC1080_BatteryStatus* batterySatus);

HDC1080_Status HDC1080_GetManufacturerNumber(uint16_t* manufacturerNumber);
HDC1080_Status HDC1080_GetDeviceId(uint16_t* deviceId);
HDC1080_Status HDC1080_GetSerialNumber(uint64_t* serialNumber);

#endif
