#include "HDC1080.h"
#include "HDC1080_PlatformSpecific.h"

#include <stdint.h>
#include <stddef.h>

#include "sam.h"

HDC1080_Status HDC1080_PlatformSpecific_Init() {
	HDC1080_SERCOM_EN_REG |= HDC1080_SERCOM_EN_BIT;

	// PIN config
	#if (HDC1080_SDA_PIN & 0x01)
	PORT->Group[HDC1080_SDA_GROUP].PMUX[HDC1080_SDA_PIN >> 1].bit.PMUXO = HDC1080_SDA_PMUX;
	#else
	PORT->Group[HDC1080_SDA_GROUP].PMUX[HDC1080_SDA_PIN >> 1].bit.PMUXE = HDC1080_SDA_PMUX;
	#endif

	#if (HDC1080_SCL_PIN & 0x01)
	PORT->Group[HDC1080_SCL_GROUP].PMUX[HDC1080_SCL_PIN >> 1].bit.PMUXO = HDC1080_SCL_PMUX;
	#else
	PORT->Group[HDC1080_SCL_GROUP].PMUX[HDC1080_SCL_PIN >> 1].bit.PMUXE = HDC1080_SCL_PMUX;
	#endif

	PORT->Group[HDC1080_SDA_GROUP].PINCFG[HDC1080_SDA_PIN].bit.PMUXEN = 1;
	PORT->Group[HDC1080_SCL_GROUP].PINCFG[HDC1080_SCL_PIN].bit.PMUXEN = 1;

	// CLOCK config
	while (GCLK->SYNCBUSY.reg) {}
	GCLK->GENCTRL[HDC1080_GCLK_GEN_NUM].reg = GCLK_GENCTRL_DIV(1) | GCLK_GENCTRL_GENEN | GCLK_GENCTRL_SRC(6);
	while (GCLK->SYNCBUSY.reg) {}
	GCLK->PCHCTRL[HDC1080_SERCOM_PCLK_NUM].reg = GCLK_PCHCTRL_CHEN | GCLK_PCHCTRL_GEN(HDC1080_GCLK_GEN_NUM);

	// SERCOM config
	HDC1080_SERCOM->I2CM.CTRLA.reg = 0;
	while (HDC1080_SERCOM->I2CM.SYNCBUSY.reg) {}

	HDC1080_SERCOM->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SWRST;
	while (HDC1080_SERCOM->I2CM.SYNCBUSY.reg) {}

	HDC1080_SERCOM->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_MODE(0x05) | SERCOM_I2CM_CTRLA_INACTOUT(0x1);
	while (HDC1080_SERCOM->I2CM.SYNCBUSY.reg) {}

	// 48 MHz / 100 kHz / 2 = 48000000 / 100000 / 2
	HDC1080_SERCOM->I2CM.BAUD.reg = SERCOM_I2CM_BAUD_BAUD(240) | SERCOM_I2CM_BAUD_HSBAUD(240);
	while (HDC1080_SERCOM->I2CM.SYNCBUSY.reg) {}

	HDC1080_SERCOM->I2CM.CTRLA.reg |= SERCOM_I2CM_CTRLA_ENABLE;
	while (HDC1080_SERCOM->I2CM.SYNCBUSY.reg) {}

	return HDC1080_Status_Ok;
}

HDC1080_Status HDC1080_PlatformSpecific_Deinit() {
	return HDC1080_Status_Ok;
}

static HDC1080_Status HDC1080_CheckI2CError() {
	uint32_t flags = HDC1080_SERCOM->I2CM.INTFLAG.reg;
	uint32_t status = HDC1080_SERCOM->I2CM.STATUS.reg;

	if (status & SERCOM_I2CM_STATUS_ARBLOST) {
		HDC1080_SERCOM->I2CM.STATUS.reg = SERCOM_I2CM_STATUS_ARBLOST;
		return HDC1080_Status_I2CArbitrationLost;
	}

	if (status & SERCOM_I2CM_STATUS_RXNACK) {
		HDC1080_SERCOM->I2CM.STATUS.reg = SERCOM_I2CM_STATUS_RXNACK;
		return HDC1080_Status_I2CNack;
	}

	if (status & SERCOM_I2CM_STATUS_BUSERR) {
		HDC1080_SERCOM->I2CM.STATUS.reg = SERCOM_I2CM_STATUS_BUSERR;
		return HDC1080_Status_I2CError;
	}

	if (flags & SERCOM_I2CM_INTFLAG_ERROR) {
		HDC1080_SERCOM->I2CM.INTFLAG.reg = SERCOM_I2CM_INTFLAG_ERROR;
		return HDC1080_Status_I2CError;

	}

	return HDC1080_Status_Ok;
}

HDC1080_Status HDC1080_PlatformSpecific_Write(uint8_t registerAddress, uint8_t* buffer, size_t bufferSize) {
	HDC1080_Status status;

	while (HDC1080_SERCOM->I2CM.STATUS.bit.BUSSTATE != 1) {}

	HDC1080_SERCOM->I2CM.ADDR.reg = HDC1080_I2C_WRITE_ADDRESS;
	while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
	status = HDC1080_CheckI2CError();
	if (status) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
		return status;
	}


	HDC1080_SERCOM->I2CM.DATA.reg = registerAddress;
	while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
	status = HDC1080_CheckI2CError();
	if (status) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
		return status;
	}

	while (bufferSize--) {
		HDC1080_SERCOM->I2CM.DATA.reg = *buffer++;
		while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
		status = HDC1080_CheckI2CError();
		if (status) {
			HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
			return status;
		}
	}

	HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);

	return HDC1080_Status_Ok;
}

HDC1080_Status HDC1080_PlatformSpecific_BeginRead(uint8_t registerAddress) {
	HDC1080_Status status;

	while (HDC1080_SERCOM->I2CM.STATUS.bit.BUSSTATE != 1) {}

	HDC1080_SERCOM->I2CM.ADDR.reg = HDC1080_I2C_WRITE_ADDRESS;
	while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
	status = HDC1080_CheckI2CError();
	if (status) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
		return status;
	}

	HDC1080_SERCOM->I2CM.DATA.reg = registerAddress;
	while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
	status = HDC1080_CheckI2CError();
	if (status) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
		return status;
	}

	return HDC1080_Status_Ok;
}

HDC1080_Status HDC1080_PlatformSpecific_EndRead(uint8_t* buffer, size_t bufferSize) {
	HDC1080_Status status;

	if (bufferSize == 0) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
		return 0;
	}

	HDC1080_SERCOM->I2CM.CTRLB.reg = 0;
	HDC1080_SERCOM->I2CM.ADDR.reg = HDC1080_I2C_READ_ADDRESS;
	while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
	status = HDC1080_CheckI2CError();
	if (status) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
		return status;
	}
	*buffer++ = HDC1080_SERCOM->I2CM.DATA.reg;
	bufferSize--;

	while (bufferSize--) {
		HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x02);
		while (HDC1080_SERCOM->I2CM.INTFLAG.reg == 0) {}
		status = HDC1080_CheckI2CError();
		if (status) {
			HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03);
			return status;
		}

		*buffer++ = HDC1080_SERCOM->I2CM.DATA.reg;
	}

	HDC1080_SERCOM->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_CMD(0x03) | SERCOM_I2CM_CTRLB_ACKACT;

	return HDC1080_Status_Ok;
}
