#include "UART.h"

#include <stddef.h>
#include <string.h>
#include <stdlib.h>

#include <sam.h>

void UART_Init() {
	REG_MCLK_APBDMASK |= MCLK_APBDMASK_SERCOM5;
	
	PORT->Group[1].PMUX[8].bit.PMUXE = 0x02;
	PORT->Group[1].PMUX[8].bit.PMUXO = 0x02;
	PORT->Group[1].PINCFG[16].bit.PMUXEN = 1;
	PORT->Group[1].PINCFG[17].bit.PMUXEN = 1;
	
	while (REG_GCLK_SYNCBUSY & GCLK_SYNCBUSY_GENCTRL11) {}
	REG_GCLK_GENCTRL11 = GCLK_GENCTRL_DIV(1) | GCLK_GENCTRL_GENEN | GCLK_GENCTRL_SRC(6);
	
	GCLK->PCHCTRL[35].reg = GCLK_PCHCTRL_CHEN | GCLK_PCHCTRL_GEN(11);
	
	while (REG_SERCOM5_USART_SYNCBUSY) {}
		
	REG_SERCOM5_USART_CTRLA = 0;
	while (REG_SERCOM5_USART_SYNCBUSY) {}
		
	REG_SERCOM5_USART_CTRLA = SERCOM_USART_CTRLA_SWRST;
	while (REG_SERCOM5_USART_SYNCBUSY) {}
		
	REG_SERCOM5_USART_CTRLA = SERCOM_USART_CTRLA_DORD | SERCOM_USART_CTRLA_MODE(1) | SERCOM_USART_CTRLA_RXPO(1);
	while (REG_SERCOM5_USART_SYNCBUSY) {}
		
	REG_SERCOM5_USART_CTRLB = SERCOM_USART_CTRLB_TXEN | SERCOM_USART_CTRLB_RXEN;
	while (REG_SERCOM5_USART_SYNCBUSY) {}
		
	REG_SERCOM5_USART_BAUD = 63019;
	
	REG_SERCOM5_USART_CTRLA |= SERCOM_USART_CTRLA_ENABLE;
	while (REG_SERCOM5_USART_SYNCBUSY) {}
	
	// UART_SetFontColor(UART_COLOR_DEFAULT);
	// UART_SetBackgroundColor(UART_COLOR_DEFAULT);
}

char UART_ReadCharBlocking() {
	REG_SERCOM5_USART_INTFLAG = SERCOM_USART_INTFLAG_RXC;
	while (!(REG_SERCOM5_USART_INTFLAG & SERCOM_SPI_INTFLAG_RXC));
	return (char)REG_SERCOM5_USART_DATA;
}

void UART_PrintChar(char ch) {
	REG_SERCOM5_USART_DATA = ch;
	while (!(REG_SERCOM5_USART_INTFLAG & SERCOM_SPI_INTFLAG_TXC));
	REG_SERCOM5_USART_INTFLAG |= SERCOM_USART_INTFLAG_TXC;
}

void UART_PrintString(char* message) {
	size_t len = strlen(message);
	for (int i = 0; i < len; i++) {
		UART_PrintChar(message[i]);
	}
}

void UART_PrintNumber(int number) {
	char buff[12];
	itoa(number, buff, 10);
	UART_PrintString(buff);
}

void UART_PrintNumber2(int number) {
	char buff[12];
	itoa(number, buff, 10);
	
	size_t len  = strlen(buff);
	while (len++ < 2) {
		UART_PrintString("0");
	}
	
	UART_PrintString(buff);
}

void UART_PrintNumberHex(int number) {
	char buff[9];
	itoa(number, buff, 16);
	UART_PrintString(buff);
}

void UART_PrintNumberHex2(int number) {
	char buff[9];
	itoa(number, buff, 16);
	if (strlen(buff) == 1) {
		UART_PrintString("0");
	}
	UART_PrintString(buff);
}

void UART_PrintNumberHex4(int number) {
	char buff[9];
	itoa(number, buff, 16);
	int len = strlen(buff);
	
	if (len == 1) {
		UART_PrintString("000");
	} else if (len == 2) {
		UART_PrintString("00");
	} else if (len == 3) {
		UART_PrintString("0");
	}
	
	UART_PrintString(buff);
}

void UART_PrintNumberBin(int number) {
	char buff[33];
	itoa(number, buff, 2);
	UART_PrintString(buff);
}

void UART_SetFontColor(uint8_t color) {
	UART_PrintString("\033[1;3");
	UART_PrintNumber(color);
	UART_PrintString("m");
}

void UART_SetBackgroundColor(uint8_t color) {
	UART_PrintNumber(color);
	UART_PrintString("m");
}

void UART_PrintFloat2(float number) {
	if (number < 0) {
		UART_PrintString("-");
		number = -number;
	}
	int number100 = (int)(number * 100);
	int beforeDot = number100 / 100;
	int afterDot = number100 % 100;
	
	UART_PrintNumber(beforeDot);
	UART_PrintString(".");
	UART_PrintNumber2(afterDot);
}
