#include "sam.h"

#include "UART.h"
#include "HDC1080.h"

#include <stdio.h>

#define PIN_BTNL 22
#define PIN_BTNR 31
#define PIN_BTNC 15

static void checkError(HDC1080_Status hStatus, char* fce) {
	if (hStatus) {
		UART_PrintString(fce);
		UART_PrintString(" failed: ");
		UART_PrintNumber(hStatus);
		UART_PrintString("\r\n");
	
		for (volatile int i = 0; i < 100000; i++) {
		}
		
		__DSB();
		SCB->AIRCR  = ((0x5FA << SCB_AIRCR_VECTKEY_Pos) | (SCB->AIRCR & SCB_AIRCR_PRIGROUP_Msk) | SCB_AIRCR_SYSRESETREQ_Msk);
		__DSB();
		while (1) {}
	}
}

static void printSensorData() {
	HDC1080_Status hStatus;
	
	hStatus = HDC1080_TriggerTemperatureAndHumidityConversion();
	checkError(hStatus, "HDC1080_TriggerConversion");
	
	for (volatile int i = 0; i < 50000; i++) {}
	
	float temp = 0;
	float humi = 0;
	hStatus = HDC1080_GetTemperatureAndHumidity(&temp, &humi);
	checkError(hStatus, "HDC1080_Get");
	
	UART_PrintString("TEMP: ");
	UART_PrintFloat2(temp);
	UART_PrintString(" degC\tHUMI: ");
	UART_PrintFloat2(humi);
	UART_PrintString(" %\r\n");
}

#define PRESS_SHORT
#define PRESS_LONG
#define PRESS_VERY_LONG

static void shortPress(uint32_t mask) {
	UART_PrintString(".");
	PORT->Group[1].OUTSET.reg = mask;
	for (volatile int i = 0; i < 1000000; i++) {}
	PORT->Group[1].OUTCLR.reg = mask;
	UART_PrintString(".");
	for (volatile int i = 0; i < 1000000; i++) {}
	UART_PrintString(" done\r\n");
}

static void longPress(uint32_t mask) {
	UART_PrintString(".");
	PORT->Group[1].OUTSET.reg = mask;
	for (volatile int i = 0; i < 8000000; i++) {}
	PORT->Group[1].OUTCLR.reg = mask;
	UART_PrintString(".");
	for (volatile int i = 0; i < 1000000; i++) {}
	UART_PrintString(" done\r\n");
}

static void veryLongPress(uint32_t mask) {
	UART_PrintString(".");
	PORT->Group[1].OUTSET.reg = mask;
	for (volatile int i = 0; i < 16000000; i++) {}
	PORT->Group[1].OUTCLR.reg = mask;
	UART_PrintString(".");
	for (volatile int i = 0; i < 1000000; i++) {}
	UART_PrintString(" done\r\n");
}


static void togglePower() {
	UART_PrintString("Toggle power: ");
	longPress(1 << PIN_BTNC);
}

static void togglePlayback() {
	UART_PrintString("Toggle playback: ");
	shortPress(1 << PIN_BTNC);
}

static void enterParing() {
	UART_PrintString("Enter pairing: ");
	veryLongPress(1 << PIN_BTNC);
}

static void volumeUp() {
	UART_PrintString("Volume up: ");
	shortPress(1 << PIN_BTNR);
}

static void volumeDown() {
	UART_PrintString("Volume down: ");
	shortPress(1 << PIN_BTNL);
}

static void songNext() {
	UART_PrintString("Next song: ");
	longPress(1 << PIN_BTNR);
}

static void songPrev() {
	UART_PrintString("Prev song: ");
	longPress(1 << PIN_BTNL);
}

int main(void)
{
	HDC1080_Status hStatus;
	
	SystemInit();
	UART_Init();
	UART_PrintString("\r\nINIT OK\r\n");
	
	hStatus = HDC1080_Init();
	checkError(hStatus, "HDC1080_Init");
	
	HDC1080_Configuration config;
	HDC1080_GetDefaultConfiguration(&config);
	hStatus = HDC1080_SetConfiguration(&config);
	checkError(hStatus, "HDC1080_SetConfiguration");
	
	// Init earbugs control pins
	PORT->Group[1].OUTCLR.reg = (1 << PIN_BTNL);
	PORT->Group[1].OUTCLR.reg = (1 << PIN_BTNR);
	PORT->Group[1].OUTCLR.reg = (1 << PIN_BTNC);
	PORT->Group[1].DIRSET.reg = (1 << PIN_BTNL);
	PORT->Group[1].DIRSET.reg = (1 << PIN_BTNR);
	PORT->Group[1].DIRSET.reg = (1 << PIN_BTNC);
	
    while (1) 
    {
		char buff[4];
		buff[0] = UART_ReadCharBlocking();
		buff[1] = '\r';
		buff[2] = '\n';
		buff[3] = '\0';
		
		UART_PrintString(buff);
		
		if (buff[0] == 's') {
			printSensorData();
		} else if (buff[0] == 'x') {
			togglePower();
		}  else if (buff[0] == 'e') {
			togglePlayback();
		} else if (buff[0] == 'c') {
			enterParing();
		} else if (buff[0] == '+') {
			volumeUp();
		} else if (buff[0] == '-') {
			volumeDown();
		} else if (buff[0] == 'n') {
			songNext();
		} else if (buff[0] == 'p') {
			songPrev();
		}
    }
}
